@@ -1,26 +1,20 @@
 # type: ignore
 
-from . import parlaclarin
-from .corpus_index import CorpusSourceIndex, CorpusSourceItem
+from . import metadata
+from .corpus.corpus_index import CorpusSourceIndex, CorpusSourceItem
+from .corpus.iterate import ProtocolSegment, ProtocolSegmentIterator
+from .corpus.parlaclarin import pretokenize
 from .dehyphenation import SwedishDehyphenator, SwedishDehyphenatorService
-from .interface import (
-    ParlaClarinError,
-    Protocol,
-    ProtocolSegment,
-    ProtocolSegmentIterator,
-    SegmentLevel,
-    Speech,
-    Utterance,
-)
-from .merge import MergedSegmentGroup, SegmentCategoryClosed, SegmentMerger, create_grouping_hashcoder
-from .parlaclarin import compute_term_frequencies, pretokenize
-from .tag import ITagger, TaggedDocument, tag_protocol, tag_protocol_xml
+from .dispatch import DispatchItem, SegmentMerger, create_grouping_hashcoder
+from .interface import ParlaClarinError, Protocol, SegmentLevel, Speech, Utterance
+from .to_speech import MergerFactory, MergeStrategyType, to_speeches
 from .utility import (
     compose,
     data_path_ts,
     dedent,
     deprecated,
     dict_get_by_path,
+    download_protocols,
     download_url,
     ensure_path,
     flatten,
@@ -37,6 +31,7 @@ from .utility import (
     path_add_timestamp,
     sanitize,
     slugify,
+    split_properties_by_dataclass,
     store_dict,
     store_token_set,
     strip_extensions,
@@ -49,3 +44,5 @@ from .utility import (
     ts_data_path,
     unlink,
 )
+from .workflows import compute_term_frequencies, extract_corpus_tags, extract_corpus_text
+from .workflows.tag import ITagger, TaggedDocument, tag_protocol, tag_protocol_xml
