@@ -4,9 +4,9 @@ from typing import Sequence
 
 import click
 
-from pyriksprot import dispatch
+from pyriksprot.dispatch import dispatch
 from pyriksprot.interface import GroupingKey, SegmentLevel, TemporalKey
-from pyriksprot.parlaclarin import extract
+from pyriksprot.workflows import extract_text
 
 from .utils import option2, update_arguments_from_options_file
 
@@ -25,6 +25,7 @@ Extract an aggregated subset of ParlaCLARIN XML corpus.
 
 @click.command()
 @click.argument('source-folder', type=click.STRING)
+@click.argument('metadata-filename', type=click.STRING)
 @click.argument('target-name', type=click.STRING)
 @option2('--options-filename')
 @option2('--target-type')
@@ -42,6 +43,7 @@ Extract an aggregated subset of ParlaCLARIN XML corpus.
 def main(
     options_filename: str = None,
     source_folder: str = None,
+    metadata_filename: str = None,
     target_name: str = None,
     target_type: str = None,
     compress_type: str = "zip",
@@ -61,7 +63,7 @@ def main(
         arguments['compress_type'] = dispatch.CompressType(arguments['compress_type'].lower())
         arguments['group_keys'] = arguments['group_key']
         del arguments['group_key']
-        extract.extract_corpus_text(**arguments)
+        extract_text.extract_corpus_text(**arguments)
 
     except Exception as ex:
         click.echo(ex)
