@@ -19,6 +19,7 @@ import unicodedata
 import warnings
 import zlib
 from collections import defaultdict
+from dataclasses import fields, is_dataclass
 from itertools import chain
 from os.path import basename, dirname, expanduser, isfile
 from os.path import join as jj
@@ -28,6 +29,7 @@ from typing import Any, Callable, List, Literal, Sequence, Set, Type, TypeVar
 from urllib.error import URLError
 from urllib.request import urlretrieve
 
+import requests
 import unidecode  # pylint: disable=import-error
 import yaml
 from loguru import logger
@@ -49,7 +51,6 @@ class dotdict(dict):
     __delattr__ = dict.__delitem__
 
 
-# FIXME: sync_delta_names hangs
 def sync_delta_names(
     source_folder: str, source_extension: str, target_folder: str, target_extension: str, delete: bool = False
 ) -> Set(str):
@@ -60,7 +61,6 @@ def sync_delta_names(
 
     delta_names = set(target_names).difference(set(source_names))
 
-    # FIXME: Move files if not delete
     if delete:
         for name in delta_names:
             path = jj(target_folder, f"{name}.{target_extension}")
@@ -235,6 +235,20 @@ def download_url(*, url: str, target_folder: str, filename: str = None) -> None:
             urlretrieve(url.replace('https:', 'http:'), target_filename)
 
 
+def download_url_to_file(url: str, target_name: str, force: bool = False) -> None:
+
+    if os.path.isfile(target_name):
+        if not force:
+            raise ValueError("File exists, use `force=True` to overwrite")
+        os.unlink(target_name)
+
+    ensure_path(target_name)
+
+    with open(target_name, 'w', encoding="utf-8") as fp:
+        data: str = requests.get(url, allow_redirects=True).content.decode("utf-8")
+        fp.write(data)
+
+
 def deprecated(func):
     """Decorator that marks functions or classes as deprecated (emits a warning when used)."""
 
@@ -380,12 +394,12 @@ def merge_tagged_csv(csv_strings: List[str], sep: str = '\n') -> str:
     texts: List[str] = [csv_strings[0]]
     for csv_string in csv_strings[1:]:
         text = strip_csv_header(csv_string, sep=sep)
-        if text != '':
+        if text:
             texts.append(text)
     return sep.join(texts)
 
 
-def store_str(filename: str, text: str, compress_type: Literal['plain', 'gzip', 'bz2', 'lzma']) -> None:
+def store_str(filename: str, text: str, compress_type: Literal['csv', 'gzip', 'bz2', 'lzma']) -> None:
     """Stores a textfile on disk - optionally compressed"""
     modules = {'gzip': (gzip, 'gz'), 'bz2': (bz2, 'bz2'), 'lzma': (lzma, 'xz')}
 
@@ -394,7 +408,7 @@ def store_str(filename: str, text: str, compress_type: Literal['plain', 'gzip',
         with module.open(f"{filename}.{extension}", 'wb') as fp:
             fp.write(text.encode('utf-8'))
 
-    elif compress_type == 'plain':
+    elif compress_type == 'csv':
         with open(filename, 'w', encoding='utf-8') as fp:
             fp.write(text)
     else:
@@ -431,3 +445,63 @@ def update_dict_from_yaml(yaml_file: str, data: dict) -> dict:
     options: dict = read_yaml(yaml_file)
     data.update(options)
     return data
+
+
+def download_protocols(*, protocols: List[str], target_folder: str, create_subfolder: bool, tag: str) -> None:
+    """Downloads protocols. Used only in tests."""
+
+    def _protocol_uri(filename: str, sub_folder: str, tag: str) -> str:
+        return f"https://github.com/welfare-state-analytics/riksdagen-corpus/raw/{tag}/corpus/protocols/{sub_folder}/{filename}"
+
+    shutil.rmtree(target_folder, ignore_errors=True)
+    os.makedirs(target_folder, exist_ok=True)
+
+    logger.info(f"downloading protocols from branch {tag}.")
+
+    for filename in protocols:
+        sub_folder: str = filename.split('-')[1]
+        filename: str = replace_extension(filename, 'xml')
+        download_url(
+            url=_protocol_uri(filename=filename, sub_folder=sub_folder, tag=tag),
+            target_folder=target_folder if not create_subfolder else jj(target_folder, sub_folder),
+            filename=filename,
+        )
+
+
+def probe_filename(filename: list[str], exts: list[str] = None) -> str | None:
+    """Probes existence of filename with any of given extensions in folder"""
+    for probe_name in set([filename] + ([replace_extension(filename, ext) for ext in exts] if exts else [])):
+        if isfile(probe_name):
+            return probe_name
+    raise FileNotFoundError(filename)
+
+
+def revdict(d: dict) -> dict:
+    return {v: k for k, v in d.items()}
+
+
+def split_properties_by_dataclass(properties: set[str], *cls_list: tuple[Type, ...]) -> tuple[set[str], ...]:
+
+    properties = set(properties)
+
+    key_sets: list[set[str]] = []
+
+    for cls in cls_list:
+
+        if not is_dataclass(cls):
+            raise ValueError(f"{cls.__name__} is not a dataclass")
+
+        key_set: set[str] = properties.intersection({f.name for f in fields(cls)})
+
+        properties -= key_set
+
+        key_sets.append(key_set)
+
+    if properties:
+        raise ValueError(f"split_properties_by_dataclass: {','.join(properties)} not found.")
+
+    return tuple(key_sets)
+
+
+def props(cls: Type) -> list[str]:
+    return [i for i in cls.__dict__.keys() if i[:1] != '_']
